/*
Steering Behaviors For Autonomous Characters by Craig W. Reynolds: http://www.red3d.com/cwr/steer/gdc99/

Three more steering behaviors animations.
You cycle through them with the right key and quit with on.
You can freeze the screen with the left key, press the down key to go on.

*/

#include <hpstdlib.h>
#include <hpkeyb49.h>
#include <hpmath.h>
#include <hpgraphics.h>
#include <hpstdio.h> 
#include <saturn.h>

#define DEGTORAD 0.017453293
#define NEIGHBORSRADIUS 8.0

#define WIDTH 131
#define HEIGHT 80

#define AGENTS  25

#define PURSUITAGENTS 5
#define WALLAGENTS 8
#define REPULSEAGENTS 25

#define MAXWALLS 8

//..................................................................VECTORS 

typedef struct
{ float x; float y; } _2dvector;

float _2dvecDot(_2dvector *v1, _2dvector *v2);
void  _2dvecNormalize(_2dvector *v);
int   _2dvecSign(_2dvector *v1, _2dvector *V2);
float _2dvecSquareDistance(_2dvector *v1, _2dvector *v2);
void _2dvecNormal(_2dvector *v1);
int lineintersection(_2dvector *v1, _2dvector *v2, _2dvector *v3, _2dvector *v4, float *distance, _2dvector *point);

//..................................................................AGENTS

_2dvector AgentShape[3] = { {5.0,0.0}, {-5.0,3.0}, {-5.0,-3.0} };

typedef struct
{
	_2dvector shape[3];						// values initialized in InitialiseAgents()
	_2dvector heading;
	float boundingradius;
	
	_2dvector position;						// values initialized in behavior function
	float rotation;
	_2dvector scale;
} _agent;

_agent Agents[AGENTS];

void InitialiseAgents(int);
void draw_agent(_agent *agent);

//..................................................................GLOBALS

_2dvector screencenter = { WIDTH/2.0 , HEIGHT/2.0 };
hpg_t *background;

void nooverlap(int, int);
void wraparound(int);
void towardcenter(int);
void steeringbehaviors(void);

typedef struct
{
	_2dvector target;
	float radius;
	float distance;
	float speed;
	int wanderagents;
}_wanderinfo;

_wanderinfo WanderAgents[AGENTS];
void SetWanderParameters(int);
void wander(int);

typedef struct
{
	_2dvector *target;
	int agentindex;
	float agentspeed;
}_seekinfo;

void seek(_seekinfo *);
void flee(_seekinfo *);

typedef struct
{
	int inneighborood;
}_flockinginfo;

_flockinginfo FlockingAgents[AGENTS];

void tagneighbors(int, int);
void cohesion(_2dvector *, int);
void alignement(_2dvector *, int);
void separation(_2dvector *, int);

//..................................................................BEHAVIORS 

int Pursuit(void);

typedef struct
{
	_2dvector from;
	_2dvector to;
}_wall;

typedef struct
{
	int wallsnumber;
	_wall walls[MAXWALLS];
	
	_2dvector normal;
	float penetrationlength;
	int agentindex;
}_wallsinfo;

void wallavoidance(_wallsinfo *, int);
int Wall(void);

int Repulse(void);

//..................................................................MAIN

int main(void)
{
	unsigned char ticks = sys_setRTCTickPeriod(1);
	background = hpg_alloc_mono_image(131, 80);
	
	if(background == 0) {
		sat_stack_push_string("Error: insufficient memory!");
		return 0;
	}
	
	hpg_blit(hpg_stdscreen, 0, 0, 131, 80, background, 0, 0);
	
	hpg_set_mode_mono(1);
	srand(0);
	steeringbehaviors();
	sys_setRTCTickPeriod(ticks); 
	hpg_free_image(background);
	return 0;
}

void steeringbehaviors(void)
{
	for(;;) {
		if(Pursuit()) return;
		if(Wall()) return;
		if(Repulse()) return;
	}
}

int Repulse()
{
	_wallsinfo wallsinfo; wallsinfo.wallsnumber = 8;
	_2dvector masscenter, separationforce, averageheading;
	_seekinfo seekinfo;

	wallsinfo.walls[0].from.x = -40.0;	 wallsinfo.walls[0].from.y = 40.0;   wallsinfo.walls[0].to.x = 40.0;   wallsinfo.walls[0].to.y = 40.0;
	wallsinfo.walls[1].from.x = 65.0;   wallsinfo.walls[1].from.y = 25.0;   wallsinfo.walls[1].to.x = 65.0;   wallsinfo.walls[1].to.y = -25.0;
	wallsinfo.walls[2].from.x = 40.0;   wallsinfo.walls[2].from.y = -39.0;  wallsinfo.walls[2].to.x = -40.0;  wallsinfo.walls[2].to.y = -39.0;
	wallsinfo.walls[3].from.x = -65.0;  wallsinfo.walls[3].from.y = -25.0;  wallsinfo.walls[3].to.x = -65.0;  wallsinfo.walls[3].to.y = 25.0;
	wallsinfo.walls[4].from.x = 40.0;	 wallsinfo.walls[4].from.y = 40.0;   wallsinfo.walls[4].to.x = 65.0;   wallsinfo.walls[4].to.y = 25.0;
	wallsinfo.walls[5].from.x = 65.0;   wallsinfo.walls[5].from.y = -25.0;  wallsinfo.walls[5].to.x = 40.0;   wallsinfo.walls[5].to.y = -39.0;
	wallsinfo.walls[6].from.x = -40.0;  wallsinfo.walls[6].from.y = -39.0;  wallsinfo.walls[6].to.x = -65.0;  wallsinfo.walls[6].to.y = -25.0;
	wallsinfo.walls[7].from.x = -65.0;  wallsinfo.walls[7].from.y = 25.0;   wallsinfo.walls[7].to.x = -40.0;  wallsinfo.walls[7].to.y = 40.0;
	
	Agents[0].scale.x = 1.0;   Agents[0].scale.y = 1.0;	
	Agents[0].position.x = 0.0; Agents[0].position.y = 0.0; Agents[0].rotation = 0.0;
	WanderAgents[0].speed = 8.0; WanderAgents[0].distance = 20.0; WanderAgents[0].radius = 20.0;	
	WanderAgents[0].target.x = WanderAgents[0].radius * cos(DEGTORAD * 20.0);
	WanderAgents[0].target.y = WanderAgents[0].radius * sin(DEGTORAD * 20.0);
	
	int i;
	for( i = 1; i < REPULSEAGENTS; i++ ) {
		Agents[i].scale.x = 0.3; Agents[i].scale.y = 0.3; 
		Agents[i].position.x = (float) (rand() % 80) - 40; 
		Agents[i].position.y = (float) (rand() %  40) - 20; 
		Agents[i].rotation = (float) (rand() % 360) * DEGTORAD;
	}
	InitialiseAgents(REPULSEAGENTS);
	
	for(;;) {
		sys_slowOff(); hpg_clear();
		hpg_blit (background, 0, 0, 131, 80, hpg_stdscreen, 0, 0);
		
		wander(0); wallsinfo.agentindex = 0; wallavoidance(&wallsinfo, 0);
		if( wallsinfo.normal.x != 0.0 || wallsinfo.normal.y != 0.0 ) {
			Agents[0].position.x -= wallsinfo.normal.x * 2.0; Agents[0].position.y -= wallsinfo.normal.y * 2.0;
			Agents[0].rotation = M_PI / 1.5;
			WanderAgents[0].target.x *= -1.5; WanderAgents[0].target.y *= -1.5;
		}
		 draw_agent(&Agents[0]);	
		 
		for( i = 1; i < REPULSEAGENTS; i++ ) {
			tagneighbors(REPULSEAGENTS, i);			
			
			separation(&separationforce, i);
			Agents[i].position.x += separationforce.x * 0.5; Agents[i].position.y += separationforce.y * 0.5;
			
			cohesion(&masscenter, i); 
			Agents[i].position.x += masscenter.x * 0.02; Agents[i].position.y += masscenter.y * 0.02;
			
			alignement(&averageheading, i);
			if(averageheading.x == 0 && averageheading.y == 0) {
				averageheading.x = Agents[i].heading.x; averageheading.y = Agents[i].heading.y;
			}			
			Agents[i].position.x += averageheading.x * 2.0; Agents[i].position.y += averageheading.y * 2.0;
			_2dvecNormalize(&averageheading);
			float angle = _2dvecSign(&Agents[i].heading,&averageheading) * acos(_2dvecDot(&Agents[i].heading, &averageheading) );
			_fabs(angle) > 0.01 ? (Agents[i].rotation = angle/2.0) : (Agents[i].rotation = 0.0);
			
			if(	_2dvecSquareDistance(&Agents[0].position, &Agents[i].position) < 1000.0) {
				seekinfo.target = &Agents[0].position;
				seekinfo.agentspeed = 1.5; seekinfo.agentindex = i; flee(&seekinfo);
			}
			wraparound(i); draw_agent(&Agents[i]);	
		}
		sys_slowOn(); hpg_flip(); sys_waitRTCTicks(2);
		if(keyb_isLeft()) while(!keyb_isDown());	
		if(keyb_isRight()) return 0;
		if(keyb_isON()) return 1;
	}
}

int Wall()
{
	_wallsinfo wallsinfo;
	wallsinfo.wallsnumber = 4;
	
	wallsinfo.walls[0].from.x = -60.0;	wallsinfo.walls[0].from.y = 35.0;  wallsinfo.walls[0].to.x = 60.0;  wallsinfo.walls[0].to.y = 35.0;
	wallsinfo.walls[1].from.x = 58.0;  wallsinfo.walls[1].from.y = 39.0;  wallsinfo.walls[1].to.x = 58.0;  wallsinfo.walls[1].to.y = -39.0;
	wallsinfo.walls[2].from.x = 60.0;  wallsinfo.walls[2].from.y = -35.0; wallsinfo.walls[2].to.x = -60.0; wallsinfo.walls[2].to.y = -35.0;
	wallsinfo.walls[3].from.x = -58.0; wallsinfo.walls[3].from.y = -39.0; wallsinfo.walls[3].to.x = -58.0; wallsinfo.walls[3].to.y = 39.0;
	
	SetWanderParameters(AGENTS);
	int i;
	for( i = 0; i < WALLAGENTS; i++ ) {
		Agents[i].scale.x = 0.5; Agents[i].scale.y = 0.5; 
		Agents[i].position.x = (float) (rand() % 80) - 40; 
		Agents[i].position.y = (float) (rand() %  40) - 20; 
		Agents[i].rotation = (float) (rand() % 360) * DEGTORAD;
		WanderAgents[i].speed = (float) (rand() % 4) + 8.0;
	}
	InitialiseAgents(WALLAGENTS);
	
	for(;;) {
		sys_slowOff(); hpg_clear();
		hpg_blit (background, 0, 0, 131, 80, hpg_stdscreen, 0, 0);
		
		for( i = 0; i < WALLAGENTS; i++ ) {
			wander(i); wallsinfo.agentindex = i; wallavoidance(&wallsinfo, 1);
			
			if( wallsinfo.normal.x != 0.0 || wallsinfo.normal.y != 0.0 ) {
				Agents[i].position.x -= wallsinfo.normal.x * 3.0;
				Agents[i].position.y -= wallsinfo.normal.y * 3.0;
				WanderAgents[i].target.x *= -1; WanderAgents[i].target.y *= -1;
			}
			if( Agents[i].position.x > WIDTH/2 || Agents[i].position.x < - (WIDTH/2) ||
				Agents[i].position.y < - (HEIGHT/2)|| Agents[i].position.y > HEIGHT/2) {
				Agents[i].position.x = 0.0; Agents[i].position.y = 0.0;
			}
			nooverlap(WALLAGENTS, i); draw_agent(&Agents[i]);	
		}
		sys_slowOn(); hpg_flip(); sys_waitRTCTicks(6);
		if(keyb_isLeft()) while(!keyb_isDown());	
		if(keyb_isRight()) return 0;
		if(keyb_isON()) return 1;
	}
}

int Pursuit()
{
	_seekinfo seekinfo;
	_2dvector center;
	center.x = center.y = 0.0;
	
	Agents[0].scale.x = 1.0;   Agents[0].scale.y = 1.0;	
	Agents[0].position.x = 0.0; Agents[0].position.y = 0.0; Agents[0].rotation = 0.0;
	int i;
	for( i = 1; i < PURSUITAGENTS; i++ ) {
		Agents[i].scale.x = 0.5; Agents[i].scale.y = 0.5;
		Agents[i].position.x = (float) (rand() % 120) - 60; 
		Agents[i].position.y = (float) (rand() %  70) - 35; 
		Agents[i].rotation = (float) (rand() % 360) * DEGTORAD;
	}
	SetWanderParameters(PURSUITAGENTS); InitialiseAgents(PURSUITAGENTS);
	
	for(;;) {
		sys_slowOff(); hpg_clear();
		hpg_blit (background, 0, 0, 131, 80, hpg_stdscreen, 0, 0);
		
		float minval = _2dvecSquareDistance(&Agents[0].position, &Agents[1].position);
		int j = 1;
		for( i = 2; i < PURSUITAGENTS; i++ ) {
			float val = _2dvecSquareDistance(&Agents[0].position, &Agents[i].position);
			if( val < minval ) { minval = val; j = i; }
		}
		if(	_2dvecSquareDistance(&Agents[0].position, &Agents[j].position) >= 3000.0) {
			seekinfo.target = &center;
			seekinfo.agentspeed = (float) (rand() % 11) + 10.0;
			seekinfo.agentindex = 0;
			seek(&seekinfo); draw_agent(&Agents[0]);
			for(i = 1; i < PURSUITAGENTS; i++) {
				wander(i); wraparound(i);
				nooverlap(PURSUITAGENTS, i); draw_agent(&Agents[i]);	
			}
		}
		else {	
			seekinfo.target = &Agents[j].position;
			seekinfo.agentspeed = (float) (rand() % 11) + 10.0;
			seekinfo.agentindex = 0;
			seek(&seekinfo); draw_agent(&Agents[0]);	
			for(i = 1; i < PURSUITAGENTS; i++) {
				wander(i); wraparound(i);
				nooverlap(PURSUITAGENTS, i); draw_agent(&Agents[i]);	
			}	
		}
		sys_slowOn(); hpg_flip(); sys_waitRTCTicks(6);
		if(keyb_isLeft()) while(!keyb_isDown());	
		if(keyb_isRight()) return 0;
		if(keyb_isON()) return 1;
	}
}

//..................................................................AGENTS FUNCTIONS

void InitialiseAgents(int agents)
{
	int i,j;
	for(i = 0; i < agents; i++) {		
		for (j=0 ; j<3 ;j++) {	
			Agents[i].shape[j].x = ((AgentShape[j].x * Agents[i].scale.x) * cos(Agents[i].rotation)) - 
									((AgentShape[j].y * Agents[i].scale.x) * sin(Agents[i].rotation));
			
			Agents[i].shape[j].y = ((AgentShape[j].x * Agents[i].scale.y) * sin(Agents[i].rotation)) + 
									((AgentShape[j].y * Agents[i].scale.y) * cos(Agents[i].rotation));		
		}
		Agents[i].boundingradius = AgentShape[0].x * Agents[i].scale.x + 1.5;
		Agents[i].heading.x = Agents[i].shape[0].x; Agents[i].heading.y = -Agents[i].shape[0].y; 
		_2dvecNormalize(&Agents[i].heading);
	}
}

void draw_agent(_agent *agent)
{
	int agent2dx[3], agent2dy[3];
	_2dvector as;
	int i;
	for (i=0 ; i<3 ;i++) {	
		as.x = (agent->shape[i].x * cos(agent->rotation)) - (agent->shape[i].y * sin(agent->rotation));
		as.y = (agent->shape[i].x * sin(agent->rotation)) + (agent->shape[i].y * cos(agent->rotation));
		agent->shape[i].x = as.x; agent->shape[i].y = as.y;
		agent2dx[i] = (int) screencenter.x + agent->position.x + as.x;
		agent2dy[i] = (int) screencenter.y - agent->position.y + as.y;
	}
	agent->heading.x = agent->shape[0].x; agent->heading.y = -agent->shape[0].y;
	_2dvecNormalize(&agent->heading);
	hpg_fill_polygon(agent2dx, agent2dy, 3);
}

//..................................................................UTILITY FUNCTIONS

void towardcenter(int index)
{
	_seekinfo seekinfo;
	_2dvector center; 
	center.x = center.y = 0.0; 
	if ( Agents[index].position.x > WIDTH/2 || Agents[index].position.x < - (WIDTH/2) ||
         Agents[index].position.y < - (HEIGHT/2)|| Agents[index].position.y > HEIGHT/2) {
		seekinfo.target = &center; seekinfo.agentspeed = 10.0;
		seekinfo.agentindex = index; seek(&seekinfo);
	}	
}

void wraparound(int index)
{
	if (Agents[index].position.x > WIDTH/2)		Agents[index].position.x = - (WIDTH/2);
	if (Agents[index].position.x < - (WIDTH/2))	Agents[index].position.x = WIDTH/2;
	if (Agents[index].position.y < - (HEIGHT/2))	Agents[index].position.y = HEIGHT/2;
	if (Agents[index].position.y > HEIGHT/2)		Agents[index].position.y = - (HEIGHT/2);
}

void nooverlap(int agents, int index)
{
	_2dvector delta;
	int i;	
	for(i = 0; i < agents; i++) {
		if(i == index) continue;
		delta.x = Agents[index].position.x - Agents[i].position.x; 	
		delta.y = Agents[index].position.y - Agents[i].position.y; 
		float distance = sqrt( (delta.x * delta.x) + (delta.y * delta.y) );
		float overlap = Agents[index].boundingradius + Agents[i].boundingradius - distance;
		if( overlap > 0 && distance > 0) {	
			Agents[index].position.x += ( delta.x / distance ) * overlap;
			Agents[index].position.y += ( delta.y / distance ) * overlap;			
		}		
	}
}

//..................................................................GROUP BEHAVIORS

void tagneighbors(int agents, int index)
{
	float range = (NEIGHBORSRADIUS + Agents[1].boundingradius) * (NEIGHBORSRADIUS + Agents[1].boundingradius);
	_2dvector distance;
	int i;
	for(i = 1; i < AGENTS; i++) {
		if(i == index) continue;
		FlockingAgents[i].inneighborood = 0;
		distance.x = Agents[index].position.x - Agents[i].position.x; 	
		distance.y = Agents[index].position.y - Agents[i].position.y; 
		float length = (distance.x * distance.x) + (distance.y * distance.y);
		if( length < range ) FlockingAgents[i].inneighborood = 1;
	}
}

void cohesion(_2dvector *masscenter, int index)
{
	int i, count = 0;
	masscenter->x = masscenter->y = 0.0;
	for(i = 1; i < AGENTS; i++) {
		if(i == index || FlockingAgents[i].inneighborood == 0) continue;
		masscenter->x += Agents[i].position.x; masscenter->y += Agents[i].position.y;
		++count;
	}
	if( count > 0 ) {
		masscenter->x /= count; masscenter->y /= count;
		masscenter->x -= Agents[index].position.x;
		masscenter->y -= Agents[index].position.y; 
	}
}

void alignement(_2dvector *averageheading, int index)
{
	int i, count = 0;
	averageheading->x = averageheading->y = 0.0;
	for(i = 1; i < AGENTS; i++) {
		if(i == index || FlockingAgents[i].inneighborood == 0) continue;
		averageheading->x += Agents[i].heading.x;
		averageheading->y += Agents[i].heading.y;
		++count;
	}
	if( count > 0 ) {
		averageheading->x /= count; averageheading->y /= count;
	}
}

void separation(_2dvector *separation, int index)
{
	_2dvector position;
	separation->x = separation->y = 0.0;
	int i;	
	for(i = 1; i < AGENTS; i++) {
		if(i == index || FlockingAgents[i].inneighborood == 0) continue;
		
		position.x = Agents[index].position.x - Agents[i].position.x;
		position.y = Agents[index].position.y - Agents[i].position.y;
		
		float length = sqrt( (position.x * position.x) + (position.y * position.y) );
		_2dvecNormalize(&position);
		
		separation->x += position.x / length;
		separation->y += position.y / length;   
	}
}

//..................................................................BEHAVIORS 

void seek(_seekinfo *seekinfo)
{
	_2dvector totarget;
	totarget.x = seekinfo->target->x - Agents[seekinfo->agentindex].position.x;
	totarget.y = seekinfo->target->y - Agents[seekinfo->agentindex].position.y;
	
	Agents[seekinfo->agentindex].position.x += totarget.x * (seekinfo->agentspeed / 100.0);
	Agents[seekinfo->agentindex].position.y += totarget.y * (seekinfo->agentspeed / 100.0);
	_2dvecNormalize(&totarget);
	
	float angle = _2dvecSign(&Agents[seekinfo->agentindex].heading, &totarget) * acos(_2dvecDot(&Agents[seekinfo->agentindex].heading, &totarget));
	_fabs(angle) > 0.01 ? (Agents[seekinfo->agentindex].rotation = angle/2.0) : (Agents[seekinfo->agentindex].rotation = 0.0);
}


void flee(_seekinfo *seekinfo)
{
	_2dvector totarget;

	totarget.x = Agents[seekinfo->agentindex].position.x - seekinfo->target->x;
	totarget.y = Agents[seekinfo->agentindex].position.y - seekinfo->target->y;

	Agents[seekinfo->agentindex].position.x += totarget.x * (seekinfo->agentspeed / 100.0);
	Agents[seekinfo->agentindex].position.y += totarget.y * (seekinfo->agentspeed / 100.0);
	
	_2dvecNormalize(&totarget);	
	float angle = _2dvecSign(&Agents[seekinfo->agentindex].heading, &totarget) * acos(_2dvecDot(&Agents[seekinfo->agentindex].heading, &totarget));
	_fabs(angle) > 0.01 ? (Agents[seekinfo->agentindex].rotation = angle/2.0) : (Agents[seekinfo->agentindex].rotation = 0.0);
}


void wander(int index)
{
	_2dvector totarget;
	
	WanderAgents[index].target.x += (float) (rand() % 21) - 10;
	WanderAgents[index].target.y += (float) (rand() % 21) - 10;
	_2dvecNormalize(&WanderAgents[index].target);
	WanderAgents[index].target.x *= WanderAgents[index].radius;
	WanderAgents[index].target.y *= WanderAgents[index].radius;
	
	totarget.x = WanderAgents[index].target.x + (Agents[index].heading.x * WanderAgents[index].distance);
	totarget.y = WanderAgents[index].target.y + (Agents[index].heading.y * WanderAgents[index].distance);
	
	Agents[index].position.x += totarget.x * ( WanderAgents[index].speed / 100.0 );
	Agents[index].position.y += totarget.y * ( WanderAgents[index].speed / 100.0 );
	
	_2dvecNormalize(&totarget);
	float angle = _2dvecSign(&Agents[index].heading, &totarget) * acos(_2dvecDot(&Agents[index].heading, &totarget) );
	_fabs(angle) > 0.01 ? (Agents[index].rotation = angle) : (Agents[index].rotation = 0.0);
}

void SetWanderParameters(int agents)
{
	int i;
	for(i = 0; i < agents; i++) {
		float theta = (float) (rand() % 360) * DEGTORAD;
		WanderAgents[i].speed = (float) (rand() % 8) + 8.0;
		WanderAgents[i].distance = (float) (rand() % 20) + 5.0;
		WanderAgents[i].radius = (float) (rand() % 20) + 10.0;	
		WanderAgents[i].target.x = WanderAgents[i].radius * cos(theta);
		WanderAgents[i].target.y = WanderAgents[i].radius * sin(theta);
	}
}

void wallavoidance(_wallsinfo *wallsinfo, int drawantennas)
{
	_2dvector antennas[3];
	float antennalength = 10.0;
	
	antennas[0].x = Agents[wallsinfo->agentindex].position.x + (antennalength * Agents[wallsinfo->agentindex].heading.x);
	antennas[0].y = Agents[wallsinfo->agentindex].position.y + (antennalength * Agents[wallsinfo->agentindex].heading.y);

	float angle = M_PI_2 * 3.5;
	_2dvector v;
	
	v.x = (Agents[wallsinfo->agentindex].heading.x * cos(angle)) - (Agents[wallsinfo->agentindex].heading.y * sin(angle));
	v.y = (Agents[wallsinfo->agentindex].heading.x * sin(angle)) + (Agents[wallsinfo->agentindex].heading.y * cos(angle));
	antennas[1].x = Agents[wallsinfo->agentindex].position.x + ( (antennalength/2.0) * v.x);
	antennas[1].y = Agents[wallsinfo->agentindex].position.y + ( (antennalength/2.0) * v.y);

	angle = M_PI_2 * 0.5;
	v.x = (Agents[wallsinfo->agentindex].heading.x * cos(angle)) - (Agents[wallsinfo->agentindex].heading.y * sin(angle));
	v.y = (Agents[wallsinfo->agentindex].heading.x * sin(angle)) + (Agents[wallsinfo->agentindex].heading.y * cos(angle));
	antennas[2].x = Agents[wallsinfo->agentindex].position.x + ( (antennalength/2.0) * v.x);
	antennas[2].y = Agents[wallsinfo->agentindex].position.y + ( (antennalength/2.0) * v.y);
	
	if(drawantennas) {
		hpg_draw_line(  (int) screencenter.x + Agents[wallsinfo->agentindex].position.x, 
						(int) screencenter.y - Agents[wallsinfo->agentindex].position.y, 
						(int) screencenter.x + antennas[0].x, (int) screencenter.y - antennas[0].y);
	
		hpg_draw_line(  (int) screencenter.x + Agents[wallsinfo->agentindex].position.x, 
						(int) screencenter.y - Agents[wallsinfo->agentindex].position.y, 
						(int) screencenter.x + antennas[1].x, (int) screencenter.y - antennas[1].y);
	
		hpg_draw_line(  (int) screencenter.x + Agents[wallsinfo->agentindex].position.x, 
						(int) screencenter.y - Agents[wallsinfo->agentindex].position.y, 
						(int) screencenter.x + antennas[2].x, (int) screencenter.y - antennas[2].y);
	}
	
	wallsinfo->normal.x = wallsinfo->normal.y = 0.0;
	float distanceIP = 0.0;
	float distanceclosestIP = 1000.0;
	_2dvector point,closestpoint,p1,p2;
	point.x = point.y = closestpoint.x = closestpoint.y = 0.0;
	int i,j,wallindex;
	
	for (i = 0; i < 3; i++) {
		wallindex = -1;
		for (j = 0; j < wallsinfo->wallsnumber; j++) {
			if (lineintersection(&Agents[wallsinfo->agentindex].position, &antennas[i], 
								&wallsinfo->walls[j].from, &wallsinfo->walls[j].to,
								&distanceIP,&point) ) {
				if (distanceIP < distanceclosestIP) {
					distanceclosestIP = distanceIP; wallindex = j;
					closestpoint.x = point.x; closestpoint.y = point.y;
				}
			}
		}
		if (wallindex >= 0) {
      		p1.x = antennas[i].x - closestpoint.x; p1.y = antennas[i].y - closestpoint.y;
			wallsinfo->penetrationlength = sqrt( (p1.x * p1.x) + (p1.y * p1.y) );
			
			p2.x = wallsinfo->walls[wallindex].to.x - wallsinfo->walls[wallindex].from.x;
			p2.y = wallsinfo->walls[wallindex].to.y - wallsinfo->walls[wallindex].from.y;
			
			_2dvecNormal(&p2);	wallsinfo->normal.x = p2.x; wallsinfo->normal.y = p2.y; 		
		}
	}
}


int lineintersection(_2dvector *v1, _2dvector *v2, _2dvector *v3, _2dvector *v4, float *distance, _2dvector *point)
{
	float numerator1  = (v1->y - v3->y) * (v4->x - v3->x) - (v1->x - v3->x) * (v4->y - v3->y);
	float numerator2  = (v1->y - v3->y) * (v2->x - v1->x) - (v1->x - v3->x) * (v2->y - v1->y);
	float denominator = (v2->x - v1->x) * (v4->y - v3->y) - (v2->y - v1->y) * (v4->x - v3->x);	
	
	if (denominator == 0) return 0;
	float a = numerator1 / denominator; float b = numerator2 / denominator;

	if( (a > 0) && (a < 1) && (b > 0) && (b < 1) ) {
		*distance = sqrt( _2dvecSquareDistance(v1,v2) ) * a;
		point->x = v1->x + (a * ( v2->x - v1->x ) );
		point->y = v1->y + (a * ( v2->y - v1->y ) );
		return 1;
	}
	else {
		*distance = 0;
		return 0;
	}
}

//..................................................................VECTORS FUNCTIONS

float _2dvecDot(_2dvector *v1, _2dvector *v2)
{
	return (v1->x * v2->x) + (v1->y * v2->y);
}

void _2dvecNormalize(_2dvector *v)
{
	float length = sqrt( (v->x * v->x) + (v->y * v->y) );
	
	if( length > 0 )
	{ v->x /= length; v->y /= length;}
}

int _2dvecSign(_2dvector *v1, _2dvector *v2)
{
	if (v1->y * v2->x > v1->x * v2->y) return 1;
	else  return -1;
}

float _2dvecSquareDistance(_2dvector *v1, _2dvector *v2)
{
	float x = v2->x - v1->x; 
	float y = v2->y - v1->y;
	return x * x + y * y;
}

void _2dvecNormal(_2dvector *v1)
  {
	_2dvecNormalize(v1);
    float x = v1->x;
	v1->x = -v1->y;
    v1->y = x;
  }


	